{"filter":false,"title":"lrucache.js","tooltip":"/src/lrucache.js","undoManager":{"mark":9,"position":9,"stack":[[{"start":{"row":0,"column":0},"end":{"row":99,"column":2},"action":"insert","lines":["function LRUCache (limit) {","  // Current size of the cache. (Read-only).","  this.size = 0;","  // Maximum number of items this cache can hold.","  this.limit = limit;","  this._keymap = {};","}","","/**"," * Put <value> into the cache associated with <key>. Returns the entry which was"," * removed to make room for the new entry. Otherwise undefined is returned"," * (i.e. if there was enough room already)."," */","LRUCache.prototype.put = function(key, value) {","  var entry = {key:key, value:value};","  // Note: No protection agains replacing, and thus orphan entries. By design.","  this._keymap[key] = entry;","  if (this.tail) {","    // link previous tail to the new tail (entry)","    this.tail.newer = entry;","    entry.older = this.tail;","  } else {","    // we're first in -- yay","    this.head = entry;","  }","  // add new entry to the end of the linked list -- it's now the freshest entry.","  this.tail = entry;","  if (this.size === this.limit) {","    // we hit the limit -- remove the head","    return this.shift();","  } else {","    // increase the size counter","    this.size++;","  }","};","","/**"," * Purge the least recently used (oldest) entry from the cache. Returns the"," * removed entry or undefined if the cache was empty."," *"," * If you need to perform any form of finalization of purged items, this is a"," * good place to do it. Simply override/replace this function:"," *"," *   var c = new LRUCache(123);"," *   c.shift = function() {"," *     var entry = LRUCache.prototype.shift.call(this);"," *     doSomethingWith(entry);"," *     return entry;"," *   }"," */","LRUCache.prototype.shift = function() {","  // todo: handle special case when limit == 1","  var entry = this.head;","  if (entry) {","    if (this.head.newer) {","      this.head = this.head.newer;","      this.head.older = undefined;","    } else {","      this.head = undefined;","    }","    // Remove last strong reference to <entry> and remove links from the purged","    // entry being returned:","    entry.newer = entry.older = undefined;","    // delete is slow, but we need to do this to avoid uncontrollable growth:","    delete this._keymap[entry.key];","  }","  return entry;","};","","/**"," * Get and register recent use of <key>. Returns the value associated with <key>"," * or undefined if not in cache."," */","LRUCache.prototype.get = function(key, returnEntry) {","  // First, find our cache entry","  var entry = this._keymap[key];","  if (entry === undefined) return; // Not cached. Sorry.","  // As <key> was found in the cache, register it as being requested recently","  if (entry === this.tail) {","    // Already the most recenlty used entry, so no need to update the list","    return returnEntry ? entry : entry.value;","  }","  // HEAD--------------TAIL","  //   <.older   .newer>","  //  <--- add direction --","  //   A  B  C  <D>  E","  if (entry.newer) {","    if (entry === this.head)","      this.head = entry.newer;","    entry.newer.older = entry.older; // C <-- E.","  }","  if (entry.older)","    entry.older.newer = entry.newer; // C. --> E","  entry.newer = undefined; // D --x","  entry.older = this.tail; // D. --> E","  if (this.tail)","    this.tail.newer = entry; // E. <-- D","  this.tail = entry;","  return returnEntry ? entry : entry.value;","};"],"id":1}],[{"start":{"row":0,"column":17},"end":{"row":0,"column":18},"action":"remove","lines":[" "],"id":2},{"start":{"row":1,"column":0},"end":{"row":1,"column":2},"action":"insert","lines":["  "]},{"start":{"row":2,"column":2},"end":{"row":2,"column":4},"action":"insert","lines":["  "]},{"start":{"row":3,"column":0},"end":{"row":3,"column":1},"action":"insert","lines":[" "]},{"start":{"row":3,"column":3},"end":{"row":3,"column":4},"action":"insert","lines":[" "]},{"start":{"row":4,"column":2},"end":{"row":4,"column":4},"action":"insert","lines":["  "]},{"start":{"row":5,"column":0},"end":{"row":5,"column":2},"action":"insert","lines":["  "]},{"start":{"row":14,"column":2},"end":{"row":14,"column":4},"action":"insert","lines":["  "]},{"start":{"row":14,"column":17},"end":{"row":15,"column":8},"action":"insert","lines":["","        "]},{"start":{"row":15,"column":12},"end":{"row":15,"column":13},"action":"insert","lines":[" "]},{"start":{"row":15,"column":17},"end":{"row":16,"column":7},"action":"insert","lines":["","       "]},{"start":{"row":16,"column":14},"end":{"row":16,"column":15},"action":"insert","lines":[" "]},{"start":{"row":16,"column":20},"end":{"row":17,"column":4},"action":"insert","lines":["","    "]},{"start":{"row":18,"column":0},"end":{"row":18,"column":2},"action":"insert","lines":["  "]},{"start":{"row":19,"column":2},"end":{"row":19,"column":4},"action":"insert","lines":["  "]},{"start":{"row":20,"column":2},"end":{"row":20,"column":4},"action":"insert","lines":["  "]},{"start":{"row":21,"column":0},"end":{"row":21,"column":3},"action":"insert","lines":["   "]},{"start":{"row":21,"column":7},"end":{"row":21,"column":8},"action":"insert","lines":[" "]},{"start":{"row":22,"column":4},"end":{"row":22,"column":8},"action":"insert","lines":["    "]},{"start":{"row":23,"column":0},"end":{"row":23,"column":2},"action":"insert","lines":["  "]},{"start":{"row":23,"column":6},"end":{"row":23,"column":8},"action":"insert","lines":["  "]},{"start":{"row":24,"column":2},"end":{"row":24,"column":4},"action":"insert","lines":["  "]},{"start":{"row":24,"column":5},"end":{"row":25,"column":3},"action":"insert","lines":["","   "]},{"start":{"row":26,"column":0},"end":{"row":26,"column":4},"action":"insert","lines":["    "]},{"start":{"row":27,"column":0},"end":{"row":27,"column":2},"action":"insert","lines":["  "]},{"start":{"row":27,"column":6},"end":{"row":27,"column":8},"action":"insert","lines":["  "]},{"start":{"row":28,"column":2},"end":{"row":28,"column":4},"action":"insert","lines":["  "]},{"start":{"row":29,"column":0},"end":{"row":29,"column":2},"action":"insert","lines":["  "]},{"start":{"row":30,"column":2},"end":{"row":30,"column":4},"action":"insert","lines":["  "]},{"start":{"row":31,"column":0},"end":{"row":31,"column":1},"action":"insert","lines":[" "]},{"start":{"row":31,"column":3},"end":{"row":31,"column":4},"action":"insert","lines":[" "]},{"start":{"row":32,"column":0},"end":{"row":32,"column":4},"action":"insert","lines":["    "]},{"start":{"row":33,"column":0},"end":{"row":33,"column":3},"action":"insert","lines":["   "]},{"start":{"row":33,"column":7},"end":{"row":33,"column":8},"action":"insert","lines":[" "]},{"start":{"row":34,"column":2},"end":{"row":34,"column":4},"action":"insert","lines":["  "]},{"start":{"row":34,"column":5},"end":{"row":35,"column":3},"action":"insert","lines":["","   "]},{"start":{"row":36,"column":0},"end":{"row":36,"column":2},"action":"insert","lines":["  "]},{"start":{"row":36,"column":6},"end":{"row":36,"column":8},"action":"insert","lines":["  "]},{"start":{"row":37,"column":4},"end":{"row":37,"column":8},"action":"insert","lines":["    "]},{"start":{"row":38,"column":0},"end":{"row":38,"column":2},"action":"insert","lines":["  "]},{"start":{"row":56,"column":2},"end":{"row":56,"column":4},"action":"insert","lines":["  "]},{"start":{"row":57,"column":2},"end":{"row":57,"column":4},"action":"insert","lines":["  "]},{"start":{"row":58,"column":0},"end":{"row":58,"column":2},"action":"insert","lines":["  "]},{"start":{"row":59,"column":4},"end":{"row":59,"column":8},"action":"insert","lines":["    "]},{"start":{"row":60,"column":0},"end":{"row":60,"column":1},"action":"insert","lines":[" "]},{"start":{"row":60,"column":7},"end":{"row":60,"column":12},"action":"insert","lines":["     "]},{"start":{"row":61,"column":0},"end":{"row":61,"column":6},"action":"insert","lines":["      "]},{"start":{"row":62,"column":4},"end":{"row":62,"column":8},"action":"insert","lines":["    "]},{"start":{"row":62,"column":9},"end":{"row":63,"column":7},"action":"insert","lines":["","       "]},{"start":{"row":64,"column":0},"end":{"row":64,"column":1},"action":"insert","lines":[" "]},{"start":{"row":64,"column":7},"end":{"row":64,"column":12},"action":"insert","lines":["     "]},{"start":{"row":65,"column":4},"end":{"row":65,"column":8},"action":"insert","lines":["    "]},{"start":{"row":66,"column":0},"end":{"row":66,"column":4},"action":"insert","lines":["    "]},{"start":{"row":67,"column":0},"end":{"row":67,"column":2},"action":"insert","lines":["  "]},{"start":{"row":67,"column":6},"end":{"row":67,"column":8},"action":"insert","lines":["  "]},{"start":{"row":68,"column":0},"end":{"row":68,"column":4},"action":"insert","lines":["    "]},{"start":{"row":69,"column":0},"end":{"row":69,"column":1},"action":"insert","lines":[" "]},{"start":{"row":69,"column":5},"end":{"row":69,"column":8},"action":"insert","lines":["   "]},{"start":{"row":70,"column":0},"end":{"row":70,"column":4},"action":"insert","lines":["    "]},{"start":{"row":71,"column":2},"end":{"row":71,"column":4},"action":"insert","lines":["  "]},{"start":{"row":72,"column":0},"end":{"row":72,"column":2},"action":"insert","lines":["  "]},{"start":{"row":80,"column":2},"end":{"row":80,"column":4},"action":"insert","lines":["  "]},{"start":{"row":81,"column":0},"end":{"row":81,"column":1},"action":"insert","lines":[" "]},{"start":{"row":81,"column":3},"end":{"row":81,"column":4},"action":"insert","lines":[" "]},{"start":{"row":82,"column":0},"end":{"row":82,"column":2},"action":"insert","lines":["  "]},{"start":{"row":83,"column":0},"end":{"row":83,"column":1},"action":"insert","lines":[" "]},{"start":{"row":83,"column":3},"end":{"row":83,"column":4},"action":"insert","lines":[" "]},{"start":{"row":84,"column":0},"end":{"row":84,"column":2},"action":"insert","lines":["  "]},{"start":{"row":85,"column":4},"end":{"row":85,"column":8},"action":"insert","lines":["    "]},{"start":{"row":86,"column":0},"end":{"row":86,"column":3},"action":"insert","lines":["   "]},{"start":{"row":86,"column":7},"end":{"row":86,"column":8},"action":"insert","lines":[" "]},{"start":{"row":87,"column":2},"end":{"row":87,"column":4},"action":"insert","lines":["  "]},{"start":{"row":88,"column":0},"end":{"row":88,"column":2},"action":"insert","lines":["  "]},{"start":{"row":89,"column":0},"end":{"row":89,"column":2},"action":"insert","lines":["  "]},{"start":{"row":90,"column":2},"end":{"row":90,"column":4},"action":"insert","lines":["  "]},{"start":{"row":91,"column":0},"end":{"row":91,"column":2},"action":"insert","lines":["  "]},{"start":{"row":92,"column":2},"end":{"row":92,"column":4},"action":"insert","lines":["  "]},{"start":{"row":93,"column":0},"end":{"row":93,"column":1},"action":"insert","lines":[" "]},{"start":{"row":93,"column":5},"end":{"row":93,"column":8},"action":"insert","lines":["   "]},{"start":{"row":94,"column":0},"end":{"row":94,"column":4},"action":"insert","lines":["    "]},{"start":{"row":94,"column":10},"end":{"row":94,"column":12},"action":"insert","lines":["  "]},{"start":{"row":95,"column":0},"end":{"row":95,"column":4},"action":"insert","lines":["    "]},{"start":{"row":96,"column":0},"end":{"row":96,"column":1},"action":"insert","lines":[" "]},{"start":{"row":96,"column":3},"end":{"row":96,"column":4},"action":"insert","lines":[" "]},{"start":{"row":97,"column":0},"end":{"row":97,"column":2},"action":"insert","lines":["  "]},{"start":{"row":98,"column":0},"end":{"row":98,"column":4},"action":"insert","lines":["    "]},{"start":{"row":99,"column":2},"end":{"row":99,"column":4},"action":"insert","lines":["  "]},{"start":{"row":100,"column":0},"end":{"row":100,"column":2},"action":"insert","lines":["  "]},{"start":{"row":101,"column":2},"end":{"row":101,"column":4},"action":"insert","lines":["  "]},{"start":{"row":102,"column":0},"end":{"row":102,"column":1},"action":"insert","lines":[" "]},{"start":{"row":102,"column":5},"end":{"row":102,"column":8},"action":"insert","lines":["   "]},{"start":{"row":103,"column":2},"end":{"row":103,"column":4},"action":"insert","lines":["  "]},{"start":{"row":104,"column":0},"end":{"row":104,"column":2},"action":"insert","lines":["  "]}],[{"start":{"row":1,"column":4},"end":{"row":1,"column":46},"action":"remove","lines":["// Current size of the cache. (Read-only)."],"id":4}],[{"start":{"row":2,"column":18},"end":{"row":2,"column":19},"action":"insert","lines":[" "],"id":5}],[{"start":{"row":2,"column":19},"end":{"row":2,"column":61},"action":"insert","lines":["// Current size of the cache. (Read-only)."],"id":6}],[{"start":{"row":3,"column":4},"end":{"row":3,"column":51},"action":"remove","lines":["// Maximum number of items this cache can hold."],"id":7}],[{"start":{"row":3,"column":0},"end":{"row":4,"column":0},"action":"remove","lines":["    ",""],"id":8}],[{"start":{"row":3,"column":23},"end":{"row":3,"column":24},"action":"insert","lines":[" "],"id":9}],[{"start":{"row":3,"column":24},"end":{"row":3,"column":71},"action":"insert","lines":["// Maximum number of items this cache can hold."],"id":10}],[{"start":{"row":1,"column":0},"end":{"row":2,"column":0},"action":"remove","lines":["    ",""],"id":11}]]},"ace":{"folds":[],"scrolltop":0,"scrollleft":0,"selection":{"start":{"row":7,"column":46},"end":{"row":7,"column":46},"isBackwards":false},"options":{"guessTabSize":true,"useWrapMode":false,"wrapToView":true},"firstLineState":0},"timestamp":1454274437384,"hash":"d7845c063e4f8013e0f83fd58109cfd7112880f2"}